package jp.co.intra_mart.cookbook_126918;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Locale;

import javax.imageio.ImageIO;

import jp.co.intra_mart.common.aid.jdk.java.util.LocaleUtil;
import jp.co.intra_mart.foundation.bis.documents.CellIndex;
import jp.co.intra_mart.foundation.bis.documents.FlowDocumentExportParams;
import jp.co.intra_mart.foundation.bis.documents.exception.DesignDocumentException;
import jp.co.intra_mart.foundation.bis.documents.flow.BisFlowDocumentSheetExporter;
import jp.co.intra_mart.foundation.bis.documents.flow.FlowData;
import jp.co.intra_mart.foundation.bis.documents.flow.ImwFlowDocumentSheetExporter;
import jp.co.intra_mart.foundation.workflow.exception.WorkflowApiException;
import jp.co.intra_mart.foundation.workflow.util.WorkflowImageManager;

import org.apache.poi_v3_9.ss.usermodel.Cell;
import org.apache.poi_v3_9.ss.usermodel.ClientAnchor;
import org.apache.poi_v3_9.ss.usermodel.Drawing;
import org.apache.poi_v3_9.ss.usermodel.Picture;
import org.apache.poi_v3_9.ss.usermodel.Row;
import org.apache.poi_v3_9.ss.usermodel.Sheet;
import org.apache.poi_v3_9.ss.usermodel.Workbook;

/**
 * フロー画像シートを出力するクラスです。
 */
public class FlowImageSheetExporter implements ImwFlowDocumentSheetExporter, BisFlowDocumentSheetExporter {

    /** フローID */
    private static final CellIndex FLOW_ID = new CellIndex("C3");

    /** フロー名 */
    private static final CellIndex FLOW_NAME = new CellIndex("C4");

    @Override
    public void execute(final FlowDocumentExportParams params, final FlowData data, final Workbook book) throws DesignDocumentException {
        if (book == null || data == null) {
            return;
        }

        // 該当シートがなければ処理なし
        final String sheetName = getSheetName(LocaleUtil.toLocale(params.getLocaleId()));
        final Sheet sheet = book.getSheet(sheetName);
        if (sheet == null) {
            return;
        }

        // フロー情報セット
        setFlowInfo(sheet, data);
        // フロー画像セット
        setFlowImage(book, sheet, data);

    }

    @Override
    public String getSheetName(final Locale locale) throws DesignDocumentException {
        return "Flow Image";
    }

    /**
     * フロー画像を設定します。
     */
    private void setFlowImage(final Workbook book, final Sheet sheet, final FlowData data) throws DesignDocumentException {
        try (ByteArrayOutputStream byteArrayOut = new ByteArrayOutputStream()) {
            final WorkflowImageManager imageManager = new WorkflowImageManager();

            final byte[] imageBytes = imageManager.getRouteImage(data.getRoute().getId(), data.getRoute().getVersionId());
            final BufferedImage img = ImageIO.read(new ByteArrayInputStream(imageBytes));
            ImageIO.write(img, "png", byteArrayOut);

            final Drawing drawing = sheet.createDrawingPatriarch();
            final ClientAnchor anchor = drawing.createAnchor(0, 0, 0, 0, 0, 5, 0, 0);
            anchor.setAnchorType(0);

            final int picIndex = book.addPicture(byteArrayOut.toByteArray(), Workbook.PICTURE_TYPE_PNG);
            final Picture pic = drawing.createPicture(anchor, picIndex);
            pic.resize();

        } catch (final WorkflowApiException | IOException e) {
            throw new DesignDocumentException(e);
        }
    }

    /**
     * フロー情報を設定します。
     */
    private void setFlowInfo(final Sheet sheet, final FlowData data) {
        setValue(sheet, FLOW_ID.getRowIndex(), FLOW_ID.getColumnIndex(), data.getFlow().getId());
        setValue(sheet, FLOW_NAME.getRowIndex(), FLOW_ID.getColumnIndex(), data.getFlow().getName());
    }

    /**
     * セルに値を設定します。
     */
    private void setValue(final Sheet sheet, final int rowIndex, final int columnIndex, final String cellValue) {
        final Row row = org.apache.poi_v3_9.ss.util.CellUtil.getRow(rowIndex, sheet);
        final Cell cell = org.apache.poi_v3_9.ss.util.CellUtil.getCell(row, columnIndex);
        // セル値の設定
        cell.setCellValue(cellValue);
    }

}